/**
 * Problem: editor
 * Correct solution (must be OK)
 * Author: stgatilov
 *
 * Solution: Dijkstra over states.
 * Array of buckets is used instead of heap.
 * All states are explicitly numbered as permutations.
 * Steps are: down, up, and cut-parse sequence.
 * Graph size:  (N+1)! x O(N^2)
 *
 * Note: undef A* for additional heuristic optimization.
 */

//#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

//if uncommented, turns Dijkstra into A*
//#define ASTAR

const int MAXN = 8;
const int MAXV = 362880;    //factorial(MAXN+1)
const int MAXD = 12800;

int n;
int initial[MAXN];
int wanted[MAXN];
enum Actions {
    actUp,
    actDown,
    actShiftPress,
    actShiftRelease,
    actCtrlX,
    actCtrlV,
    actCOUNT
};
int times[actCOUNT];

int EncodeState(int *order, int pos) {
    int res = pos;
    for (int i = 0; i < n; i++) {
        int k = 0;
        for (int j = 0; j < i; j++)
            k += (order[j] < order[i]);
        int dig = order[i] - k;
        res *= n - i;
        res += dig;
        assert(dig < n-i);
    }
    return res;
}
void DecodeState(int state, int *order, int &pos) {
    int digits[MAXN];
    for (int i = 0; i < n; i++) {
        digits[n-1-i] = state % (i+1);
        state /= (i+1);
    }
    pos = state;
    bool used[MAXN] = {false};
    for (int i = 0; i < n; i++) {
        int q = digits[i];
        int j;
        for (j = 0; j < n; j++) if (!used[j])
            if (--q < 0)
                break;
        assert(j < n);
        order[i] = j;
        used[j] = true;
    }
}
string StateStr(int state) {
    int order[MAXN], pos;
    DecodeState(state, order, pos);
    string res;
    for (int i = 0; i < n; i++) res += char('0' + order[i]);
    res += '|';
    res += char('0' + pos);
    return res;
}

struct CutPaste {
    int order[MAXN];
    int pos;
    int cost;
    vector<string> how;
};

int vk, v1, vstart, vend;
vector<CutPaste> cutpaste[MAXN+1];

int dist[MAXV], pos[MAXV];
int hrem[MAXV];
int fstate[MAXV], ftrans[MAXV];
vector<int> buckets[MAXD];

void AddToBucket(int u) {
    int d = dist[u];
    pos[u] = buckets[d].size();
    buckets[d].push_back(u);
}
void RemoveFromBucket(int u) {
    buckets[dist[u]][pos[u]] = -1;
}
void Transition(int p, int q, int cost, int trs) {
    int ndist = dist[p] - hrem[p] + cost + hrem[q];
    assert(ndist >= dist[p]);
    if (ndist >= dist[q])
        return;
    if (dist[q] < 1000000000)
        RemoveFromBucket(q);
    dist[q] = ndist;
    AddToBucket(q);
    fstate[q] = p;
    ftrans[q] = trs;
//    Ef("relax: D[%s] = %d   (from %s)\n", StateStr(q).c_str(), ndist, StateStr(p).c_str());
}

int main(int argc, char **argv) {
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);

    //read input
    scanf("%d", &n);
    assert(n <= MAXN);
    for (int i = 0; i < actCOUNT; i++)
        scanf("%d", &times[i]);
    for (int i = 0; i < n; i++) {
        scanf("%d", &initial[i]);
        initial[i]--;
    }
    for (int i = 0; i < n; i++) {
        scanf("%d", &wanted[i]);
        wanted[i]--;
    }

    //precompute cut/paste transitions
    for (int pos = 0; pos <= n; pos++) {
        for (int other = 0; other <= n; other++) if (other != pos) {
            int a = min(pos, other);
            int b = max(pos, other);
            for (int paste = 0; paste <= n; paste++) if (paste < a || paste > b) {
                int l = min(a, paste);
                int r = max(b, paste);
                int m = (paste < a ? a : b);
                assert(l < m && m < r);

                CutPaste cp;
                for (int i = 0; i < n; i++) cp.order[i] = i;
                for (int i = l; i < m; i++) cp.order[i + (r-m)] = i;
                for (int i = m; i < r; i++) cp.order[i - (m-l)] = i;
                for (int i = 0; i < n; i++) assert(cp.order[i] >= 0 && cp.order[i] < n);
                cp.pos = (paste < a ? paste + (b - a) : paste);
                int dist = (paste < a ? a - paste : paste - b);
                cp.cost = 0;

                cp.how.push_back("Shift-Press");
                cp.cost += times[actShiftPress];
                for (int i = 0; i < b-a; i++) {
                    cp.how.push_back(pos == b ? "Up" : "Down");
                    cp.cost += times[pos == b ? actUp : actDown];
                }
                cp.how.push_back("Shift-Release");
                cp.cost += times[actShiftRelease];
                cp.how.push_back("Ctrl+X");
                cp.cost += times[actCtrlX];
                for (int i = 0; i < dist; i++) {
                    cp.how.push_back(paste < a ? "Up" : "Down");
                    cp.cost += times[paste < a ? actUp : actDown];
                }
                cp.how.push_back("Ctrl+V");
                cp.cost += times[actCtrlV];

                cutpaste[pos].push_back(cp);
            }
        }

/*        Ef("Pos = %d\n", pos);
        for (auto cp : cutpaste[pos]) {
            for (int i = 0; i < n; i++) Ef("%d", cp.order[i]);
            Ef(" -> %d : %d\n", cp.pos, cp.cost);
        }*/
    }

    vk = 1;
    for (int i = 1; i <= n+1; i++)
        vk *= i;
    v1 = vk / (n+1);
    memset(dist, 63, vk * sizeof(dist[0]));
    memset(pos, -1, vk * sizeof(pos[0]));

#ifdef ASTAR
    for (int u = 0; u < vk; u++) {
        int order[MAXN], pos;
        DecodeState(u, order, pos);
        int wrong = 0;
        for (int i = 0; i < n; i++)
            wrong += (order[i] != wanted[i]);
        if (wrong > 0) {
            int inv[MAXN];
            for (int i = 0; i < n; i++)
                inv[wanted[i]] = i;
            int splits = 0;
            for (int i = 0; i <= n; i++) {
                int prev = (i == 0 ? -1 : inv[order[i-1]]);
                int next = (i == n ? n : inv[order[i]]);
                splits += (next != prev+1);
            }
            //we have to sweep through the dirty elements
            int minSweep = wrong;
            //every cut fixes at least 3 splits
            int minCuts = (splits + 2) / 3;

            hrem[u] = minSweep * min(times[actUp], times[actDown]) + minCuts * (times[actShiftPress] + times[actShiftRelease] + times[actCtrlX] + times[actCtrlV]);
        }
    }
#endif

    vstart = EncodeState(initial, 0);
    vend = EncodeState(wanted, 0);
    dist[vstart] = hrem[vstart];
    fstate[vstart] = -1;
    AddToBucket(vstart);

    //run Dijkstra
    int term = -1;
    int processed = 0;
    for (int layer = 0; term < 0 && layer < MAXD; layer++) {
        while (buckets[layer].size()) {
            int p = buckets[layer].back();
            buckets[layer].pop_back();
            if (p < 0) continue;

            int order[MAXN];
            int pos;
            DecodeState(p, order, pos);

            if (p - pos * v1 == vend) {
                term = p;
                break;
            }

            for (int s = -1; s <= 1; s += 2) {
                int npos = pos + s;
                if (npos < 0 || npos > n)
                    continue;
                int q = EncodeState(order, npos);
                Transition(p, q, times[s < 0 ? actUp : actDown], s-2);
            }

            for (int j = 0; j < cutpaste[pos].size(); j++) {
                const auto &cp = cutpaste[pos][j];
                int norder[MAXN];
                for (int i = 0; i < n; i++)
                    norder[i] = order[cp.order[i]];
                int q = EncodeState(norder, cp.pos);
                Transition(p, q, cp.cost, j);
            }

            processed++;
        }
    }
    Ef("Visited: %d / %d\n", processed, vk);
    assert(term >= 0);

    vector<string> seq;
    int curr = term;
    while (fstate[curr] >= 0) {
        int prev = fstate[curr];
        int trs = ftrans[curr];
        if (trs < 0) {
            int s = trs + 2;
            seq.push_back(s < 0 ? "Up" : "Down");
        }
        else {
            int order[MAXN], pos;
            DecodeState(prev, order, pos);
            const CutPaste &cp = cutpaste[pos][trs];
            for (int j = cp.how.size()-1; j >= 0; j--)
                seq.push_back(cp.how[j]);
        }
        curr = prev;
    }
    reverse(seq.begin(), seq.end());

    printf("%d %d\n", dist[term], int(seq.size()));
    for (auto s : seq)
        printf("%s\n", s.c_str());

    return 0;
}
